@@ -277,11 +277,19 @@ def get_procedures_plot(procedures: list[Procedure]):
     Arguments:
         procedures: A list of procedures to be executed.
     """
-    fig = make_subplots(rows=3, cols=1, shared_xaxes=True, vertical_spacing=0.1)
+    fig = make_subplots(
+        rows=3,
+        cols=1,
+        shared_xaxes=True,
+        vertical_spacing=0.1,
+        row_heights=[0.6, 0.2, 0.2],
+    )
 
     kinds = sorted(set(p.kind for p in procedures))
+    kind_to_index = {value: i for i, value in enumerate(kinds)}
     _times = sorted(set(p.time for p in procedures))
     systems = sorted(set(p.system for p in procedures))
+    system_to_index = {value: i for i, value in enumerate(systems)}
 
     procedures = sorted(
         procedures, key=lambda p: (p.kind, p.system, p.time, -p.cost, -p.duration)
@@ -296,12 +304,11 @@ def get_procedures_plot(procedures: list[Procedure]):
     xs = list(range(int(_times[0]), int(_times[-1] + 1)))
 
     heatmap_z: list[list[Optional[int]]] = [[None for j in xs] for i in systems]
-    for kind_index, kind in enumerate(kinds):
-
-        for p in procedures:
-            if p.kind != kind:
-                continue
-            heatmap_z[systems.index(p.system)][int(p.time) - xs[0]] = kind_index
+    heatmap_text: list[list[Optional[str]]] = [[None for j in xs] for i in systems]
+    for p in procedures:
+        row, col = system_to_index[p.system], int(p.time) - xs[0]
+        heatmap_z[row][col] = kind_to_index[p.kind]
+        heatmap_text[row][col] = p.kind
 
     for kind_index, kind in enumerate(kinds):
         times: list[Union[int, float]] = []
@@ -315,25 +322,29 @@ def get_procedures_plot(procedures: list[Procedure]):
             times.append(p.time)
             costs.append(p.cost)
             duras.append(p.duration)
-            objs.append(systems.index(p.system))
+            objs.append(system_to_index[p.system])
 
         # Heatmap, with adjusted colormaps to trick plotly's via legend toggles.
         hm_colors = [
             colors[kind] if kind == _kind else "rgba(0,0,0,0)" for _kind in kinds
         ]
+
         fig.add_trace(
             go.Heatmap(
                 x=xs,
                 y=systems,
                 z=heatmap_z,
                 legendgroup=kind,
-                name=kind,
+                name="procedures",
                 colorscale=hm_colors,
                 showlegend=False,
                 showscale=False,
                 hoverongaps=False,
                 xgap=2,
                 ygap=2,
+                hovertemplate="time: %{x}<br>system: %{y}<br>"
+                + "procedure: %{customdata}",
+                customdata=heatmap_text,
             )
         )
 
